//===-- AlphaISelDAGToDAG.cpp - Alpha pattern matching inst selector ------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// This file defines a pattern matching instruction selector for Alpha,
// converting from a legalized dag to a Alpha dag.
//
//===----------------------------------------------------------------------===//

#include "Z80.h"
#include "Z80TargetMachine.h"
#include "llvm/CodeGen/MachineInstrBuilder.h"
#include "llvm/CodeGen/MachineFrameInfo.h"
#include "llvm/CodeGen/MachineFunction.h"
#include "llvm/CodeGen/MachineRegisterInfo.h"
#include "llvm/CodeGen/SelectionDAG.h"
#include "llvm/CodeGen/SelectionDAGISel.h"
#include "llvm/Target/TargetOptions.h"
#include "llvm/Constants.h"
#include "llvm/DerivedTypes.h"
#include "llvm/GlobalValue.h"
#include "llvm/Intrinsics.h"
#include "llvm/LLVMContext.h"
#include "llvm/Support/Compiler.h"
#include "llvm/Support/Debug.h"
#include "llvm/Support/ErrorHandling.h"
#include "llvm/Support/MathExtras.h"
#include "llvm/Support/raw_ostream.h"
#include <algorithm>
#include <stdio.h>
using namespace llvm;

namespace {

  //===--------------------------------------------------------------------===//
  /// AlphaDAGToDAGISel - Alpha specific code to select Alpha machine
  /// instructions for SelectionDAG operations.
  class Z80DAGToDAGISel : public SelectionDAGISel {
        
    

  public:
    explicit Z80DAGToDAGISel(Z80TargetMachine &TM)
      : SelectionDAGISel(TM)
    {}


    // Select - Convert the specified operand from a target-independent to a
    // target-specific node if it hasn't already been changed.
    SDNode *Select(SDNode *N);
    
    virtual const char *getPassName() const {
      return "Z80 DAG->DAG Pattern Instruction Selection";
    } 

    inline SDValue getI16Imm(uint64_t imm) {
	return CurDAG->getTargetConstant(imm, MVT::i16);
    }


// Include the pieces autogenerated from the target description.
#include "Z80GenDAGISel.inc"
    
private:
    /// getTargetMachine - Return a reference to the TargetMachine, casted
    /// to the target-specific type.
    const Z80TargetMachine &getTargetMachine() {
      return static_cast<const Z80TargetMachine &>(TM);
    }

    /// getInstrInfo - Return a reference to the TargetInstrInfo, casted
    /// to the target-specific type.
    const Z80InstrInfo *getInstrInfo() {
      return getTargetMachine().getInstrInfo();
    }
  };
}



// Select - Convert the specified operand from a target-independent to a
// target-specific node if it hasn't already been changed.
SDNode *Z80DAGToDAGISel::Select(SDNode *N) {

   if (N->isMachineOpcode())
	return NULL;

   switch (N->getOpcode())
   {
   default:
	break;
   case ISD::FrameIndex:
	int FI = cast<FrameIndexSDNode>(N)->getIndex();
    	return CurDAG->SelectNodeTo(N, Z80::addHLdisp, MVT::i16,
                                CurDAG->getTargetFrameIndex(FI, MVT::i16),
                                getI16Imm(0));
   }   	
   return SelectCode(N);
}

/// createAlphaISelDag - This pass converts a legalized DAG into a 
/// Alpha-specific DAG, ready for instruction scheduling.
///
FunctionPass *llvm::createZ80ISelDag(Z80TargetMachine &TM) {
  return new Z80DAGToDAGISel(TM);
}
