//===- AlphaInstrInfo.cpp - Alpha Instruction Information -------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// This file contains the Alpha implementation of the TargetInstrInfo class.
//
//===----------------------------------------------------------------------===//

#include "Z80InstrInfo.h"
#include "Z80.h"
#include "Z80Subtarget.h"
#include "Z80TargetMachine.h"
#include "llvm/DerivedTypes.h"
#include "llvm/LLVMContext.h"
#include "llvm/ADT/STLExtras.h"
#include "llvm/CodeGen/MachineConstantPool.h"
#include "llvm/CodeGen/MachineFrameInfo.h"
#include "llvm/CodeGen/MachineInstrBuilder.h"
#include "llvm/CodeGen/MachineRegisterInfo.h"
#include "llvm/CodeGen/LiveVariables.h"
#include "llvm/CodeGen/PseudoSourceValue.h"
#include "llvm/MC/MCInst.h"
#include "llvm/Support/CommandLine.h"
#include "llvm/Support/Debug.h"
#include "llvm/Support/ErrorHandling.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/Target/TargetOptions.h"
#include "llvm/MC/MCAsmInfo.h"
#include <limits>


#define GET_INSTRINFO_CTOR
#include "Z80GenInstrInfo.inc"
using namespace llvm;

Z80InstrInfo::Z80InstrInfo()
  : Z80GenInstrInfo(), 
	RI(*this)
{
}

void
Z80InstrInfo::storeRegToStackSlot(MachineBasicBlock &MBB,
                                    MachineBasicBlock::iterator MI,
                                    unsigned SrcReg, bool isKill, int FrameIdx,
                                    const TargetRegisterClass *RC,
                                    const TargetRegisterInfo *TRI) const {

  DebugLoc DL;
  if (MI != MBB.end()) DL = MI->getDebugLoc();

  if (RC == Z80::GPR16RegisterClass || RC == Z80::HL16RegisterClass)
    BuildMI(MBB, MI, DL, get(Z80::STQ))
      .addReg(SrcReg, getKillRegState(isKill))
      .addFrameIndex(FrameIdx).addReg(Z80::SP);
  else {

    dbgs() << "RC is " << RC->getName() << "\n";
    dbgs() << "Trying to store " << Z80RegisterInfo::getPrettyName(SrcReg) << " to FrameIndex "
        << FrameIdx << "\n";

    llvm_unreachable("Unhandled register class");
  }
}

void
Z80InstrInfo::loadRegFromStackSlot(MachineBasicBlock &MBB,
                                        MachineBasicBlock::iterator MI,
                                        unsigned DestReg, int FrameIdx,
                                     const TargetRegisterClass *RC,
                                     const TargetRegisterInfo *TRI) const {
  DebugLoc DL;
  if (MI != MBB.end()) DL = MI->getDebugLoc();

  if (RC == Z80::GPR16RegisterClass || RC == Z80::HL16RegisterClass)
    BuildMI(MBB, MI, DL, get(Z80::LDQ), DestReg)
      .addFrameIndex(FrameIdx).addReg(Z80::SP);
  else {

    dbgs() << "RC is " << RC->getName() << "\n";
    dbgs() << "Trying to load " << Z80RegisterInfo::getPrettyName(DestReg) << " to FrameIndex "
     << FrameIdx << "\n";

    llvm_unreachable("Unhandled register class");
  }
}


void Z80InstrInfo::copyPhysReg(MachineBasicBlock &MBB,
                               MachineBasicBlock::iterator MI, DebugLoc DL,
                               unsigned DestReg, unsigned SrcReg,
                               bool KillSrc) const {

  unsigned Opc = 0;
  if (Z80::GPRRegClass.contains(DestReg, SrcReg)) {
    Opc = Z80::mov8;
  }


  if (Z80::GPR16RegClass.contains(DestReg, SrcReg)) {
    Opc = Z80::mov16;
  }

  if (Z80::GPR16RegClass.contains(DestReg) && Z80::HL16RegClass.contains(SrcReg)) {
    Opc = Z80::mov16;
  }

  if (Z80::GPR16RegClass.contains(SrcReg) && Z80::HL16RegClass.contains(DestReg)) {
    Opc = Z80::mov16;
  }


  if (Opc) {
    BuildMI(MBB, MI, DL, get(Opc), DestReg)
      .addReg(SrcReg, getKillRegState(KillSrc));
    return;
  }

  dbgs() << "Cannot copy " << RI.getName(SrcReg)
               << " to " << RI.getName(DestReg) << '\n';
  llvm_unreachable("Cannot emit physreg copy instruction");
}

